from colorama import init, Fore, Back, Style

init(convert=True)

def info(message):
    return f"{Fore.CYAN}{message}{Fore.RESET}"

def success(message):
    return f"{Fore.GREEN}{message}{Fore.RESET}"

def warning(message):
    return f"{Fore.YELLOW}{message}{Fore.RESET}"

def error(message):
    return f"{Fore.RED}{message}{Fore.RESET}"

def debug(message):
    return f"{Fore.MAGENTA}{message}{Fore.RESET}"

def disabled(message):
    return f"{Fore.LIGHTBLACK_EX}{Style.DIM}{message}{Style.RESET_ALL}"


"""
A package which contains a bunch of small helpful devtools while scripting
"""

from .argsguard import args_guard
from .parse import parse_int, parse_float
from .ostools import *
from .sysargs import args_to_dictionary
from . import colorama_ext
from .progression_bar import progress_bar


from .colorama_ext import Fore, Style
from . import ostools
from datetime import datetime
from time import time

class log:
    def __init__(self, file_location:str, prefix:str, print_to_console:bool=True, append:bool=False, warnings_enabled=True, date_time_displayed:bool=False, time_since_start:bool=True, datetime_format:str="%H:%M:%S", updates_save=False) -> None:
        """
        A function to initialise the log class, which will log given updates to a certain file.
        -> file_location*: str => This takes in the file in which logs will be stored.
        -> prefix*: str => This will be printed out in the begining of the file with all system information which will be printed by default.
        -> print_to_console => This will print to console everytime something is logged.
        -> append: bool (False) => This will append to the given file instead of overwriting the file by default.
        -> warnings_enabled: bool (True) => This will save logs which are marked as "WARNING".
        -> date_time_displayed: bool (False) => This will display a datetime stamp everytime something is logged.
        -> time_since_start: bool (True) => This will display a timestamp (seconds since logging has started) everytime something is logged.
        -> datetime_format: str ("%H:%M:%S") => If 'date_time_displayed' is enabled, the logger will use this given datetime format. (datetime module)
        -> updates_save: bool (False) => WARNING; This will slow down the program if enabled. This will immediately save all updates everytime when something is logged.
        """

        self.PRINT_TO_CONSOLE = print_to_console
        self.APPEND = append
        self.FILE_LOCATION = file_location
        self.PREFIX = prefix
        self.WARNINGS_ENABLED = warnings_enabled
        self.DATE_TIME_DISPLAYED = date_time_displayed
        self.TIME_SINCE_START = time_since_start
        self.DATETIME_FORMAT = datetime_format
        self.UPDATES_SAVE = updates_save

        self.log_content = ""

        self.start_time = 0
        self.has_started = False

    def start(self) -> None:
        """
        Starts the logging process.
        ----
        => None
        """
        print(f"{Fore.GREEN}Logging has started.{Fore.RESET}")
        self.start_time = time()
        self.has_started = True
        file_content = f"{self.PREFIX}\n[{ostools.OS} - {datetime.now()}]\n\n"
        if self.APPEND:
            with open(self.FILE_LOCATION, "a") as log_file:
                log_file.write(file_content)
        else:
            with open(self.FILE_LOCATION, "w") as log_file:
                log_file.write(file_content)

    def log(self, message:str) -> None:
        """
        Logs a message to the log file. (Generic)
        -> message: str => The message to be logged.
        ----
        => None
        """
        if not self.has_started:
            self.start()

        content = ""

        if self.DATE_TIME_DISPLAYED:
            content = f"[{datetime.now().strftime(self.DATETIME_FORMAT)}] "

        if self.TIME_SINCE_START:
            content += f"[{int(time() - self.start_time)}] {message}"

        if self.PRINT_TO_CONSOLE:
            print(content)

        if self.UPDATES_SAVE:
            with open(self.FILE_LOCATION, "a") as log_file:
                log_file.write(content + "\n")
        else:
            self.log_content += content + "\n"

    def warning(self, message:str) -> None:
        """
        Logs a warning to the log file.
        -> message: str => The message to be logged.
        ----
        => None
        """
        if not self.has_started:
            self.start()

        if self.WARNINGS_ENABLED:
            content = ""

            if self.DATE_TIME_DISPLAYED:
                content = f"[{datetime.now().strftime(self.DATETIME_FORMAT)}] "

            if self.TIME_SINCE_START:
                content += f"[{int(time() - self.start_time)}] WARNING : {message}"

            if self.PRINT_TO_CONSOLE:
                if self.WARNINGS_ENABLED:
                    print(f"{Fore.YELLOW}{content}{Fore.RESET}")
                else:
                    print(f"{Fore.LIGHTBLACK_EX}{Style.DIM}Warnings disabled.{Style.RESET_ALL}")

            if self.UPDATES_SAVE:
                with open(self.FILE_LOCATION, "a") as log_file:
                    log_file.write(content + "\n")
            else:
                self.log_content += content + "\n"

    def info(self, message:str) -> None:
        """
        Logs an info to the log file.
        -> message: str => The message to be logged.
        ----
        => None
        """
        if not self.has_started:
            self.start()

        content = ""

        if self.DATE_TIME_DISPLAYED:
            content = f"[{datetime.now().strftime(self.DATETIME_FORMAT)}] "

        if self.TIME_SINCE_START:
            content += f"[{int(time() - self.start_time)}] INFO : {message}"

        if self.PRINT_TO_CONSOLE:
            print(f"{Fore.CYAN}{content}{Fore.RESET}")

        if self.UPDATES_SAVE:
            with open(self.FILE_LOCATION, "a") as log_file:
                log_file.write(content + "\n")
        else:
            self.log_content += content + "\n"

    def success(self, message:str) -> None:
        """
        Logs a success to the log file.
        -> message: str => The message to be logged.
        ----
        => None
        """
        if not self.has_started:
            self.start()

        content = ""

        if self.DATE_TIME_DISPLAYED:
            content = f"[{datetime.now().strftime(self.DATETIME_FORMAT)}] "

        if self.TIME_SINCE_START:
            content += f"[{int(time() - self.start_time)}] SUCCESS : {message}"

        if self.PRINT_TO_CONSOLE:
            print(f"{Fore.GREEN}{content}{Fore.RESET}")

        if self.UPDATES_SAVE:
            with open(self.FILE_LOCATION, "a") as log_file:
                log_file.write(content + "\n")
        else:
            self.log_content += content + "\n"

    def error(self, message:str) -> None:
        """
        Logs an error to the log file.
        -> message: str => The message to be logged.
        ----
        => None
        """
        if not self.has_started:
            self.start()

        content = ""

        if self.DATE_TIME_DISPLAYED:
            content = f"[{datetime.now().strftime(self.DATETIME_FORMAT)}] "

        if self.TIME_SINCE_START:
            content += f"[{int(time() - self.start_time)}] ERROR : {message}"

        if self.PRINT_TO_CONSOLE:
            print(f"{Fore.RED}{content}{Fore.RESET}")

        if self.UPDATES_SAVE:
            with open(self.FILE_LOCATION, "a") as log_file:
                log_file.write(content + "\n")
        else:
            self.log_content += content + "\n"

    def end(self) -> None:
        """
        Ends the logging process.
        ----
        => None
        """
        if not self.UPDATES_SAVE:
            with open(self.FILE_LOCATION, "a") as log_file:
                log_file.write(self.log_content + "\n\n")

        print(f"{Fore.GREEN}Logging has ended.{Fore.RESET}")



import sys

SYSTEM_ARGUMENTS = sys.argv[1:]

def args_to_dictionary(default_msg:str = "", boolean_options:list=[], arguments:list=SYSTEM_ARGUMENTS) -> dict:
    """
    A function that returns a dictionary of the system arguments passed to the program.
    -> default_msg: str => A default message to be displayed when the program is called without any arguments.
    -> boolean_options: list of boolean options which will take no arguments.
    -> arguments: list of arguments to passed to the program. (defaults to sys.argv).
    ----
    => dict: A dictionary of the system arguments passed to the program.
    """
    system_args = arguments
    option = True
    options_list = []
    arguments_list = []

    if (not isinstance(boolean_options, list)):
        raise ValueError('boolean options must be a list')
    
    if (not isinstance(default_msg, str)):
        raise ValueError('default_msg must be a string')

    if not system_args:
        print(default_msg)
    else:
        for i in system_args:
            if option and i.startswith("-"):
                options_list.append(i)
                if i not in boolean_options:
                    option = False
                else:
                    arguments_list.append(True)

            elif not option:
                arguments_list.append(i)
                option = True
            else:
                raise ValueError('An option that starts with "-", must be specified')

    if not option:
        raise ValueError(f"An argument to option {system_args[-1]}, must be specified")

    return dict(zip(options_list, arguments_list))

#print(sys.argv)




def progress_bar (iteration, total, prefix = ' ', suffix = ' ', decimals = 1, length = 100, fill = '█', printEnd = "\r") -> None:
    """
    Call in a loop to create terminal progress bar
    -> iteration*: int => current iteration
    -> total*: int => total iterations
    -> prefix: str => prefix string
    -> suffix: str => suffix string
    -> decimals: int => positive number of decimals in percent complete
    -> length: int => character length of bar
    -> fill: str => bar fill character
    -> printEnd: str => end character (e.g. "\r", "\r\n")
    ----
    => None
    """

    if not isinstance(iteration, int):
        raise ValueError("iteration must be an integer")

    if not isinstance(total, int):
        raise ValueError("total must be an integer")    

    percent = ("{0:." + str(decimals) + "f}").format(100 * (iteration / float(total)))
    filledLength = int(length * iteration // total)
    bar = fill * filledLength + '-' * (length - filledLength)
    print(f'\r{prefix} |{bar}| {percent}% {suffix}', end = printEnd)
    # Print New Line on Complete
    if iteration == total: 
        print()